﻿using System;
using System.Collections.Generic;
using PcapDotNet.Core;
using PcapDotNet.Packets;
using System.Linq;

using CommandLineParser;

namespace killabit
{
    class Program
    {
        public static Dictionary<string, ulong> traffic = new Dictionary<string, ulong>();
        public static DateTime nextdisplay = new DateTime();
        public static bool by_layer2 = true;
        public static bool by_layer3 = false;
        public static bool by_layer4 = false;
        public static int seconds_to_delay = 5;
        public static DateTime startTime = new DateTime();
        public static bool combine_directions = false;
        public static int head = 0;

        static void Main(string[] args)
        {

            nextdisplay = DateTime.Now;
            startTime = DateTime.Now;

            List<CommandParserOption> pArgs = new List<CommandParserOption>();
            
            pArgs.Add(new CommandParserOption('l', "list", "list available interfaces", null));

            pArgs.Add(new CommandParserOption('i', "interface", "use specified numbered interface from list", "1", 1));
            pArgs.Add(new CommandParserOption('w', "wait", "Wait X seconds between screen prints", "5", 1));

            pArgs.Add(new CommandParserOption('2', "layer2", "collect data based on MAC address pairs", null));
            pArgs.Add(new CommandParserOption('3', "layer3", "collect data based on IP address pairs", null));
            pArgs.Add(new CommandParserOption('4', "layer4", "collect data based on IP address:port pairs", null));

            pArgs.Add(new CommandParserOption('c', "combine", "combine matching pairs (ignore direction)", null));
            pArgs.Add(new CommandParserOption('t', "top", "only display the top X pairs (by traffic)", "10", 1));

            pArgs.Add(new CommandParserOption('h', "help", null, new char[] { '?' }, "Display usage", null));
            pArgs.Add(new CommandParserOption('v', "version", "print version and exit", null));


            Parser p = new Parser(pArgs, args);

            if (p.Flag("-h") == true)
            {
                Console.WriteLine("Usage: " + System.AppDomain.CurrentDomain.FriendlyName.Replace(".exe", "") + " " + p.UsageShort());
                Console.WriteLine();
                Console.WriteLine(p.UsageLong());

                return;
            }

            if (p.Flag("version") == true)
            {
                Console.WriteLine("Killabit.exe version 0.1 by Eli Fulkerson Apr 13 2017");
                Console.WriteLine("http://www.elifulkerson.com");
                return;
            }

            if (p.Flag("layer3") == true) {
                by_layer3 = true;
                by_layer2 = false;
                by_layer4 = false;
            }

            if (p.Flag("layer2") == true)
            {
                by_layer3 = false;
                by_layer2 = true;
                by_layer4 = false;
            }

            if (p.Flag("layer4") == true)
            {
                by_layer3 = false;
                by_layer2 = false;
                by_layer4 = true;
            }

            if (p.Flag("combine") == true)
            {
                combine_directions = true;
            }

            if (p.Flag("top") == true)
            {
                head = Convert.ToInt32(p.FlagValue("top"));
            }
            

            if (p.Flag("wait") == true) {
                seconds_to_delay = Convert.ToInt32(p.FlagValue("wait"));
            }

            // Retrieve the device list from the local machine
            IList<LivePacketDevice> allDevices = LivePacketDevice.AllLocalMachine;

            if (allDevices.Count == 0)
            {
                Console.WriteLine("No interfaces found! Make sure WinPcap is installed.");
                return;
            }

            // Print the list
            for (int i = 0; i != allDevices.Count; ++i)
            {
                LivePacketDevice device = allDevices[i];
                Console.Write((i + 1) + ". " + device.Name);
                if (device.Description != null)
                    Console.WriteLine(" (" + device.Description + ")");
                else
                    Console.WriteLine(" (No description available)");
            }

            if (p.Flag("list") == true)
            {
                Environment.Exit(0);
            }

            int deviceIndex = 0;

            if (p.Flag("interface") == true)
            {
                deviceIndex = Convert.ToInt32(p.FlagValue("interface"));

            }
            else
            {
                do
                {
                    Console.WriteLine("Enter the interface number (1-" + allDevices.Count + "):");
                    string deviceIndexString = Console.ReadLine();
                    if (!int.TryParse(deviceIndexString, out deviceIndex) ||
                        deviceIndex < 1 || deviceIndex > allDevices.Count)
                    {
                        deviceIndex = 0;
                    }
                } while (deviceIndex == 0);
            }
            // Take the selected adapter
            PacketDevice selectedDevice = allDevices[deviceIndex - 1];

            // Open the device
            using (PacketCommunicator communicator =
                selectedDevice.Open(65536,                                  // portion of the packet to capture
                // 65536 guarantees that the whole packet will be captured on all the link layers
                                    PacketDeviceOpenAttributes.Promiscuous, // promiscuous mode
                                    1000))                                  // read timeout
            {
                Console.WriteLine("Listening on " + selectedDevice.Description + "...");

                // start the capture
                communicator.ReceivePackets(0, PacketHandler);
            }
        }

        // Callback function invoked by Pcap.Net for every incoming packet
        private static void PacketHandler(Packet packet)
        {
            //Console.WriteLine(packet.Timestamp.ToString("yyyy-MM-dd hh:mm:ss.fff") + " length:" + packet.Length);
            //Console.WriteLine(packet.Ethernet.Source.ToString() + "-->" + packet.Ethernet.Destination.ToString() + "..." + packet.Length);
            
            string key = "";
            

            if (by_layer2 == true)
            {
                //key = packet.Ethernet.Source.ToString() + " -> " + packet.Ethernet.Destination.ToString();
                string src = packet.Ethernet.Source.ToString();
                string dst = packet.Ethernet.Destination.ToString();
                string arr = " -> ";

                if (combine_directions == true && string.Compare(src, dst) == 1)
                {
                    string tmp = dst;
                    dst = src;
                    src = tmp;
                    arr = " <-> ";
                }

                if (combine_directions == true) { arr = " <-> "; }

                //key = packet.Ethernet.IpV4.Source.ToString() + " -> " + packet.Ethernet.IpV4.Destination.ToString();
                key = src + arr + dst;
            }

            //string key = packet.IpV4.Source.ToString() + " -> " + packet.IpV4.Destination.ToString();

            if (by_layer3 == true)
            {


                key = "neither";
                if (packet.Ethernet.EtherType == PcapDotNet.Packets.Ethernet.EthernetType.IpV4)
                {
                    string src = packet.Ethernet.IpV4.Source.ToString();
                    string dst = packet.Ethernet.IpV4.Destination.ToString();
                    string arr = " -> ";

                    if (combine_directions == true && string.Compare(src, dst) == 1)
                    {
                        string tmp = dst;
                        dst = src;
                        src = tmp;
                        arr = " <-> ";
                    }
                    if (combine_directions == true) { arr = " <-> "; }

                    //key = packet.Ethernet.IpV4.Source.ToString() + " -> " + packet.Ethernet.IpV4.Destination.ToString();
                    key = src + arr + dst;
                }
                if (packet.Ethernet.EtherType == PcapDotNet.Packets.Ethernet.EthernetType.IpV6)
                {
                    //key = packet.Ethernet.IpV6.Source.ToString() + " -> " + packet.Ethernet.IpV6.CurrentDestination.ToString();
                    
                    string src = packet.Ethernet.IpV6.Source.ToString();
                    string dst = packet.Ethernet.IpV6.CurrentDestination.ToString();
                    string arr = " -> ";
                    if (combine_directions == true && string.Compare(src, dst) == 1)
                    {
                        string tmp = dst;
                        dst = src;
                        src = tmp;
                    }
                    if (combine_directions == true) { arr = " <-> "; }

                    //key = packet.Ethernet.IpV4.Source.ToString() + " -> " + packet.Ethernet.IpV4.Destination.ToString();
                    key = src + arr + dst;
                }

                if (key == "neither")
                {
                    return;
                }
            }

            if (by_layer4 == true)
            {
                //Console.WriteLine(packet.Ethernet.IpV4.IsValid   );
                bool found = false;

                key = "key error";

                int srcport = 0;
                int dstport = 0;

                string src = "";
                string dst = "";

                string icmpcode = "";

                string ptype = "UNKNOWN";

                //Console.WriteLine(packet.Ethernet.Ip.Version.ToString());

                if (packet.Ethernet.EtherType == PcapDotNet.Packets.Ethernet.EthernetType.IpV4)
                
                {


                    src = packet.Ethernet.IpV4.Source.ToString();
                    dst = packet.Ethernet.IpV4.Destination.ToString();

                    //if (packet.Ethernet.Ip.Tcp != null)
                    //@@ fix these, they can apparently still null out when checking for sourceport/destport
                    if (packet.Ethernet.IpV4.Protocol == PcapDotNet.Packets.IpV4.IpV4Protocol.Tcp)
                    {
                        if (packet.Ethernet.IpV4.Tcp != null)
                        {
                            srcport = packet.Ethernet.IpV4.Tcp.SourcePort;
                            dstport = packet.Ethernet.IpV4.Tcp.DestinationPort;
                        }

                        ptype = "TCPv4";


                    }
                    if (packet.Ethernet.IpV4.Protocol == PcapDotNet.Packets.IpV4.IpV4Protocol.Udp)
                    {
                        if (packet.Ethernet.IpV4.Udp != null)
                        {
                            srcport = packet.Ethernet.IpV4.Udp.SourcePort;
                            dstport = packet.Ethernet.IpV4.Udp.DestinationPort;
                        }

                        src = packet.Ethernet.IpV4.Source.ToString();
                        dst = packet.Ethernet.IpV4.Destination.ToString();

                        ptype = "UDPv4";

                    }

                }


                if (packet.Ethernet.EtherType == PcapDotNet.Packets.Ethernet.EthernetType.IpV6)
                {
                    src = packet.Ethernet.IpV6.Source.ToString();
                    dst = packet.Ethernet.IpV6.CurrentDestination.ToString();

                    if (packet.Ethernet.Ip.Tcp != null)
                    {
                        srcport = packet.Ethernet.IpV6.Tcp.SourcePort;
                        dstport = packet.Ethernet.IpV6.Tcp.DestinationPort;
                        ptype = "TCPv6";

                    }
                    if (packet.Ethernet.Ip.Udp != null)
                    {
                        srcport = packet.Ethernet.IpV6.Udp.SourcePort;
                        dstport = packet.Ethernet.IpV6.Udp.DestinationPort;
                        ptype = "UDPv6";
                    }

                }

                if (ptype == "UNKNOWN")
                {
                    //Console.WriteLine(packet.Ethernet.EtherType.ToString());
                    return;
                }

                string arr = " -> ";

                if (combine_directions == true && string.Compare(src, dst) == 1)
                {
                    string tmp = dst;
                    dst = src;
                    src = tmp;

                    int itmp = dstport;
                    dstport = srcport;
                    srcport = itmp;
                    
                    arr = " <-> ";
                }
                if (combine_directions == true) { arr = " <-> "; }
                 

                if (icmpcode.Length > 0)
                {
                    key = ptype + " " + src + arr + dst + " " + icmpcode;
                }
                else
                {
                    key = ptype + " " + src + ":" + srcport + arr + dst + ":" + dstport;
                }

            }

            ulong t = 0;

            if (traffic.ContainsKey(key))
            {
                t = traffic[key];
                t += Convert.ToUInt64(packet.Length);
                traffic[key] = t;
            }
            else
            {
                traffic.Add(key, Convert.ToUInt64(packet.Length));
            }


            if (nextdisplay < DateTime.Now)
            {

                var sortedtraffic = from entry in traffic orderby entry.Value descending select entry;


                ulong total = 0;
                foreach (KeyValuePair<string, ulong> entry in sortedtraffic)
                {
                    total += entry.Value;
                }

                Console.WriteLine();
                Console.WriteLine();
                Console.WriteLine("-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-");
                Console.WriteLine("Started: " + startTime.ToLongDateString() + " " + startTime.ToLongTimeString());
                Console.WriteLine("Report at: " + (nextdisplay - startTime).Seconds + " seconds.");
                
                Console.WriteLine();

                bool firstone = true;
                int numlength = 0;
                int count = 0;
                foreach (KeyValuePair<string, ulong> entry in sortedtraffic)
                {
                    if (count == head && head != 0)
                    {
                        break;
                    }

                    if (firstone)
                    {
                        numlength = entry.Value.ToString().Length;
                        if (numlength < 5) { numlength = 5; }
                        firstone = false;
                        Console.WriteLine("bytes".PadRight(numlength) + " " + "%".PadLeft(3) + " address pair");
                        Console.WriteLine("-----".PadRight(numlength) + " " + "---".PadLeft(3) + " ------------");
                    }

                    float percent = 100 * entry.Value / total;

                    Console.WriteLine( entry.Value.ToString().PadRight(numlength) + " " + percent.ToString().PadLeft(2) + "% " + entry.Key);
                    count += 1;

                }

                nextdisplay = DateTime.Now.AddSeconds(seconds_to_delay);
            }

        }
    }
}